@@ -1,7 +1,10 @@
 """Base async HTTP client implementation."""
 
+from __future__ import annotations
+
 import sys
 from http.client import responses
+from typing import TYPE_CHECKING
 
 from vine import Thenable, maybe_promise, promise
 
@@ -10,6 +13,9 @@ from kombu.utils.compat import coro
 from kombu.utils.encoding import bytes_to_str
 from kombu.utils.functional import maybe_list, memoize
 
+if TYPE_CHECKING:
+    from types import TracebackType
+
 __all__ = ('Headers', 'Response', 'Request')
 
 PYPY = hasattr(sys, 'pypy_version_info')
@@ -38,10 +44,12 @@ class Request:
     """A HTTP Request.
 
     Arguments:
+    ---------
         url (str): The URL to request.
         method (str): The HTTP method to use (defaults to ``GET``).
 
     Keyword Arguments:
+    -----------------
         headers (Dict, ~kombu.asynchronous.http.Headers): Optional headers for
             this request
         body (str): Optional body for this request.
@@ -132,7 +140,8 @@ class Request:
 class Response:
     """HTTP Response.
 
-    Arguments:
+    Arguments
+    ---------
         request (~kombu.asynchronous.http.Request): See :attr:`request`.
         code (int): See :attr:`code`.
         headers (~kombu.asynchronous.http.Headers): See :attr:`headers`.
@@ -140,7 +149,8 @@ class Response:
         effective_url (str): See :attr:`effective_url`.
         status (str): See :attr:`status`.
 
-    Attributes:
+    Attributes
+    ----------
         request (~kombu.asynchronous.http.Request): object used to
             get this response.
         code (int): HTTP response code (e.g. 200, 404, or 500).
@@ -176,7 +186,8 @@ class Response:
     def raise_for_error(self):
         """Raise if the request resulted in an HTTP error code.
 
-        Raises:
+        Raises
+        ------
             :class:`~kombu.exceptions.HttpError`
         """
         if self.error:
@@ -187,6 +198,7 @@ class Response:
         """The full contents of the response body.
 
         Note:
+        ----
             Accessing this property will evaluate the buffer
             and subsequent accesses will be cached.
         """
@@ -253,5 +265,10 @@ class BaseClient:
     def __enter__(self):
         return self
 
-    def __exit__(self, *exc_info):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None
+    ) -> None:
         self.close()
