@@ -1,44 +1,42 @@
-"""
-kombu.messaging
-===============
+"""Sending and receiving messages."""
 
-Sending and receiving messages.
-
-"""
-from __future__ import absolute_import
-
-import numbers
+from __future__ import annotations
 
 from itertools import count
+from typing import TYPE_CHECKING
 
+from .common import maybe_declare
 from .compression import compress
-from .connection import maybe_channel, is_connection
-from .entity import Exchange, Queue, DELIVERY_MODES
+from .connection import is_connection, maybe_channel
+from .entity import Exchange, Queue, maybe_delivery_mode
 from .exceptions import ContentDisallowed
-from .five import text_t, values
 from .serialization import dumps, prepare_accept_content
-from .utils import ChannelPromise, maybe_list
+from .utils.functional import ChannelPromise, maybe_list
 
-__all__ = ['Exchange', 'Queue', 'Producer', 'Consumer']
+if TYPE_CHECKING:
+    from types import TracebackType
+
+__all__ = ('Exchange', 'Queue', 'Producer', 'Consumer')
 
 
-class Producer(object):
+class Producer:
     """Message Producer.
 
-    :param channel: Connection or channel.
-    :keyword exchange: Optional default exchange.
-    :keyword routing_key: Optional default routing key.
-    :keyword serializer: Default serializer. Default is `"json"`.
-    :keyword compression: Default compression method. Default is no
-        compression.
-    :keyword auto_declare: Automatically declare the default exchange
-      at instantiation. Default is :const:`True`.
-    :keyword on_return: Callback to call for undeliverable messages,
-        when the `mandatory` or `immediate` arguments to
-        :meth:`publish` is used. This callback needs the following
-        signature: `(exception, exchange, routing_key, message)`.
-        Note that the producer needs to drain events to use this feature.
-
+    Arguments:
+    ---------
+        channel (kombu.Connection, ChannelT): Connection or channel.
+        exchange (kombu.entity.Exchange, str): Optional default exchange.
+        routing_key (str): Optional default routing key.
+        serializer (str): Default serializer. Default is `"json"`.
+        compression (str): Default compression method.
+            Default is no compression.
+        auto_declare (bool): Automatically declare the default exchange
+            at instantiation. Default is :const:`True`.
+        on_return (Callable): Callback to call for undeliverable messages,
+            when the `mandatory` or `immediate` arguments to
+            :meth:`publish` is used. This callback needs the following
+            signature: `(exception, exchange, routing_key, message)`.
+            Note that the producer needs to drain events to use this feature.
     """
 
     #: Default exchange
@@ -53,9 +51,8 @@ class Producer(object):
     #: Default compression method.  Disabled by default.
     compression = None
 
-    #: By default the exchange is declared at instantiation.
-    #: If you want to declare manually then you can set this
-    #: to :const:`False`.
+    #: By default, if a defualt exchange is set,
+    #: that exchange will be declare when publishing a message.
     auto_declare = True
 
     #: Basic return callback.
@@ -84,7 +81,7 @@ class Producer(object):
             self.revive(self._channel)
 
     def __repr__(self):
-        return '<Producer: {0._channel}>'.format(self)
+        return f'<Producer: {self._channel}>'
 
     def __reduce__(self):
         return self.__class__, self.__reduce_args__()
@@ -96,80 +93,105 @@ class Producer(object):
     def declare(self):
         """Declare the exchange.
 
-        This happens automatically at instantiation if
-        :attr:`auto_declare` is enabled.
-
+        Note:
+        ----
+            This happens automatically at instantiation when
+            the :attr:`auto_declare` flag is enabled.
         """
         if self.exchange.name:
             self.exchange.declare()
 
     def maybe_declare(self, entity, retry=False, **retry_policy):
-        """Declare the exchange if it hasn't already been declared
-        during this session."""
+        """Declare exchange if not already declared during this session."""
         if entity:
-            from .common import maybe_declare
             return maybe_declare(entity, self.channel, retry, **retry_policy)
 
+    def _delivery_details(self, exchange, delivery_mode=None,
+                          maybe_delivery_mode=maybe_delivery_mode,
+                          Exchange=Exchange):
+        if isinstance(exchange, Exchange):
+            return exchange.name, maybe_delivery_mode(
+                delivery_mode or exchange.delivery_mode,
+            )
+        # exchange is string, so inherit the delivery
+        # mode of our default exchange.
+        return exchange, maybe_delivery_mode(
+            delivery_mode or self.exchange.delivery_mode,
+        )
+
     def publish(self, body, routing_key=None, delivery_mode=None,
                 mandatory=False, immediate=False, priority=0,
                 content_type=None, content_encoding=None, serializer=None,
                 headers=None, compression=None, exchange=None, retry=False,
-                retry_policy=None, declare=[], **properties):
+                retry_policy=None, declare=None, expiration=None, timeout=None,
+                **properties):
         """Publish message to the specified exchange.
 
-        :param body: Message body.
-        :keyword routing_key: Message routing key.
-        :keyword delivery_mode: See :attr:`delivery_mode`.
-        :keyword mandatory: Currently not supported.
-        :keyword immediate: Currently not supported.
-        :keyword priority: Message priority. A number between 0 and 9.
-        :keyword content_type: Content type. Default is auto-detect.
-        :keyword content_encoding: Content encoding. Default is auto-detect.
-        :keyword serializer: Serializer to use. Default is auto-detect.
-        :keyword compression: Compression method to use.  Default is none.
-        :keyword headers: Mapping of arbitrary headers to pass along
-          with the message body.
-        :keyword exchange: Override the exchange.  Note that this exchange
-          must have been declared.
-        :keyword declare: Optional list of required entities that must
-            have been declared before publishing the message.  The entities
-            will be declared using :func:`~kombu.common.maybe_declare`.
-        :keyword retry: Retry publishing, or declaring entities if the
-            connection is lost.
-        :keyword retry_policy: Retry configuration, this is the keywords
-            supported by :meth:`~kombu.Connection.ensure`.
-        :keyword \*\*properties: Additional message properties, see AMQP spec.
-
+        Arguments:
+        ---------
+            body (Any): Message body.
+            routing_key (str): Message routing key.
+            delivery_mode (enum): See :attr:`delivery_mode`.
+            mandatory (bool): Currently not supported.
+            immediate (bool): Currently not supported.
+            priority (int): Message priority. A number between 0 and 9.
+            content_type (str): Content type. Default is auto-detect.
+            content_encoding (str): Content encoding. Default is auto-detect.
+            serializer (str): Serializer to use. Default is auto-detect.
+            compression (str): Compression method to use.  Default is none.
+            headers (Dict): Mapping of arbitrary headers to pass along
+                with the message body.
+            exchange (kombu.entity.Exchange, str): Override the exchange.
+                Note that this exchange must have been declared.
+            declare (Sequence[EntityT]): Optional list of required entities
+                that must have been declared before publishing the message.
+                The entities will be declared using
+                :func:`~kombu.common.maybe_declare`.
+            retry (bool): Retry publishing, or declaring entities if the
+                connection is lost.
+            retry_policy (Dict): Retry configuration, this is the keywords
+                supported by :meth:`~kombu.Connection.ensure`.
+            expiration (float): A TTL in seconds can be specified per message.
+                Default is no expiration.
+            timeout (float): Set timeout to wait maximum timeout second
+                for message to publish.
+            **properties (Any): Additional message properties, see AMQP spec.
         """
+        _publish = self._publish
+
+        declare = [] if declare is None else declare
         headers = {} if headers is None else headers
         retry_policy = {} if retry_policy is None else retry_policy
         routing_key = self.routing_key if routing_key is None else routing_key
         compression = self.compression if compression is None else compression
-        exchange = exchange or self.exchange
 
-        if isinstance(exchange, Exchange):
-            delivery_mode = delivery_mode or exchange.delivery_mode
-            exchange = exchange.name
-        else:
-            delivery_mode = delivery_mode or self.exchange.delivery_mode
-        if not isinstance(delivery_mode, numbers.Integral):
-            delivery_mode = DELIVERY_MODES[delivery_mode]
-        properties['delivery_mode'] = delivery_mode
+        exchange_name, properties['delivery_mode'] = self._delivery_details(
+            exchange or self.exchange, delivery_mode,
+        )
+
+        if expiration is not None:
+            properties['expiration'] = str(int(expiration * 1000))
 
         body, content_type, content_encoding = self._prepare(
             body, serializer, content_type, content_encoding,
             compression, headers)
 
-        publish = self._publish
+        if self.auto_declare and self.exchange.name:
+            if self.exchange not in declare:
+                # XXX declare should be a Set.
+                declare.append(self.exchange)
+
         if retry:
-            publish = self.connection.ensure(self, publish, **retry_policy)
-        return publish(body, priority, content_type,
-                       content_encoding, headers, properties,
-                       routing_key, mandatory, immediate, exchange, declare)
+            _publish = self.connection.ensure(self, _publish, **retry_policy)
+        return _publish(
+            body, priority, content_type, content_encoding,
+            headers, properties, routing_key, mandatory, immediate,
+            exchange_name, declare, timeout
+        )
 
     def _publish(self, body, priority, content_type, content_encoding,
                  headers, properties, routing_key, mandatory,
-                 immediate, exchange, declare):
+                 immediate, exchange, declare, timeout=None):
         channel = self.channel
         message = channel.prepare_message(
             body, priority, content_type,
@@ -178,10 +200,16 @@ class Producer(object):
         if declare:
             maybe_declare = self.maybe_declare
             [maybe_declare(entity) for entity in declare]
+
+        # handle autogenerated queue names for reply_to
+        reply_to = properties.get('reply_to')
+        if isinstance(reply_to, Queue):
+            properties['reply_to'] = reply_to.name
         return channel.basic_publish(
             message,
             exchange=exchange, routing_key=routing_key,
             mandatory=mandatory, immediate=immediate,
+            timeout=timeout
         )
 
     def _get_channel(self):
@@ -195,6 +223,7 @@ class Producer(object):
 
     def _set_channel(self, channel):
         self._channel = channel
+
     channel = property(_get_channel, _set_channel)
 
     def revive(self, channel):
@@ -212,19 +241,21 @@ class Producer(object):
             if self.on_return:
                 self._channel.events['basic_return'].add(self.on_return)
             self.exchange = self.exchange(channel)
-        if self.auto_declare:
-            # auto_decare is not recommended as this will force
-            # evaluation of the channel.
-            self.declare()
 
     def __enter__(self):
         return self
 
-    def __exit__(self, *exc_info):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None
+    ) -> None:
         self.release()
 
     def release(self):
         pass
+
     close = release
 
     def _prepare(self, body, serializer=None, content_type=None,
@@ -238,7 +269,7 @@ class Producer(object):
         else:
             # If the programmer doesn't want us to serialize,
             # make sure content_encoding is set.
-            if isinstance(body, text_t):
+            if isinstance(body, str):
                 if not content_encoding:
                     content_encoding = 'utf-8'
                 body = body.encode(content_encoding)
@@ -261,18 +292,21 @@ class Producer(object):
             pass
 
 
-class Consumer(object):
+class Consumer:
     """Message consumer.
 
-    :param channel: see :attr:`channel`.
-    :param queues: see :attr:`queues`.
-    :keyword no_ack: see :attr:`no_ack`.
-    :keyword auto_declare: see :attr:`auto_declare`
-    :keyword callbacks: see :attr:`callbacks`.
-    :keyword on_message: See :attr:`on_message`
-    :keyword on_decode_error: see :attr:`on_decode_error`.
-
+    Arguments:
+    ---------
+        channel (kombu.Connection, ChannelT): see :attr:`channel`.
+        queues (Sequence[kombu.Queue]): see :attr:`queues`.
+        no_ack (bool): see :attr:`no_ack`.
+        auto_declare (bool): see :attr:`auto_declare`
+        callbacks (Sequence[Callable]): see :attr:`callbacks`.
+        on_message (Callable): See :attr:`on_message`
+        on_decode_error (Callable): see :attr:`on_decode_error`.
+        prefetch_count (int): see :attr:`prefetch_count`.
     """
+
     ContentDisallowed = ContentDisallowed
 
     #: The connection/channel to use for this consumer.
@@ -298,8 +332,7 @@ class Consumer(object):
     #:
     #: The signature of the callbacks must take two arguments:
     #: `(body, message)`, which is the decoded message body and
-    #: the `Message` instance (a subclass of
-    #: :class:`~kombu.transport.base.Message`).
+    #: the :class:`~kombu.Message` instance.
     callbacks = None
 
     #: Optional function called whenever a message is received.
@@ -313,8 +346,7 @@ class Consumer(object):
     #: has the ``compression`` header set.
     #:
     #: The signature of the callback must take a single argument,
-    #: which is the raw message object (a subclass of
-    #: :class:`~kombu.transport.base.Message`).
+    #: which is the :class:`~kombu.Message` object.
     #:
     #: Also note that the ``message.body`` attribute, which is the raw
     #: contents of the message body, may in some cases be a read-only
@@ -337,58 +369,82 @@ class Consumer(object):
     #: in which case only json is allowed.
     accept = None
 
-    _tags = count(1)   # global
+    #: Initial prefetch count
+    #:
+    #: If set, the consumer will set the prefetch_count QoS value at startup.
+    #: Can also be changed using :meth:`qos`.
+    prefetch_count = None
+
+    #: Mapping of queues we consume from.
+    _queues = None
+
+    _tags = count(1)  # global
 
     def __init__(self, channel, queues=None, no_ack=None, auto_declare=None,
                  callbacks=None, on_decode_error=None, on_message=None,
-                 accept=None):
+                 accept=None, prefetch_count=None, tag_prefix=None):
         self.channel = channel
-        self.queues = self.queues or [] if queues is None else queues
+        self.queues = maybe_list(queues or [])
         self.no_ack = self.no_ack if no_ack is None else no_ack
         self.callbacks = (self.callbacks or [] if callbacks is None
                           else callbacks)
         self.on_message = on_message
+        self.tag_prefix = tag_prefix
         self._active_tags = {}
         if auto_declare is not None:
             self.auto_declare = auto_declare
         if on_decode_error is not None:
             self.on_decode_error = on_decode_error
         self.accept = prepare_accept_content(accept)
+        self.prefetch_count = prefetch_count
 
         if self.channel:
             self.revive(self.channel)
 
+    @property
+    def queues(self):  # noqa
+        return list(self._queues.values())
+
+    @queues.setter
+    def queues(self, queues):
+        self._queues = {q.name: q for q in queues}
+
     def revive(self, channel):
         """Revive consumer after connection loss."""
         self._active_tags.clear()
         channel = self.channel = maybe_channel(channel)
-        self.queues = [queue(self.channel)
-                       for queue in maybe_list(self.queues)]
-        for queue in self.queues:
+        # modify dict size while iterating over it is not allowed
+        for qname, queue in list(self._queues.items()):
+            # name may have changed after declare
+            self._queues.pop(qname, None)
+            queue = self._queues[queue.name] = queue(self.channel)
             queue.revive(channel)
 
         if self.auto_declare:
             self.declare()
 
+        if self.prefetch_count is not None:
+            self.qos(prefetch_count=self.prefetch_count)
+
     def declare(self):
         """Declare queues, exchanges and bindings.
 
-        This is done automatically at instantiation if :attr:`auto_declare`
-        is set.
-
+        Note:
+        ----
+            This is done automatically at instantiation
+            when :attr:`auto_declare` is set.
         """
-        for queue in self.queues:
+        for queue in self._queues.values():
             queue.declare()
 
     def register_callback(self, callback):
-        """Register a new callback to be called when a message
-        is received.
-
-        The signature of the callback needs to accept two arguments:
-        `(body, message)`, which is the decoded message body
-        and the `Message` instance (a subclass of
-        :class:`~kombu.transport.base.Message`.
+        """Register a new callback to be called when a message is received.
 
+        Note:
+        ----
+            The signature of the callback needs to accept two arguments:
+            `(body, message)`, which is the decoded message body
+            and the :class:`~kombu.Message` instance.
         """
         self.callbacks.append(callback)
 
@@ -396,35 +452,34 @@ class Consumer(object):
         self.consume()
         return self
 
-    def __exit__(self, *exc_info):
-        try:
-            self.cancel()
-        except Exception:
-            pass
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None
+    ) -> None:
+        if self.channel and self.channel.connection:
+            conn_errors = self.channel.connection.client.connection_errors
+            if not isinstance(exc_val, conn_errors):
+                try:
+                    self.cancel()
+                except Exception:
+                    pass
 
     def add_queue(self, queue):
         """Add a queue to the list of queues to consume from.
 
-        This will not start consuming from the queue,
-        for that you will have to call :meth:`consume` after.
-
+        Note:
+        ----
+            This will not start consuming from the queue,
+            for that you will have to call :meth:`consume` after.
         """
         queue = queue(self.channel)
         if self.auto_declare:
             queue.declare()
-        self.queues.append(queue)
+        self._queues[queue.name] = queue
         return queue
 
-    def add_queue_from_dict(self, queue, **options):
-        """This method is deprecated.
-
-        Instead please use::
-
-            consumer.add_queue(Queue.from_dict(d))
-
-        """
-        return self.add_queue(Queue.from_dict(queue, **options))
-
     def consume(self, no_ack=None):
         """Start consuming messages.
 
@@ -433,13 +488,15 @@ class Consumer(object):
         it will not cancel consuming from removed queues (
         use :meth:`cancel_by_queue`).
 
-        :param no_ack: See :attr:`no_ack`.
-
+        Arguments:
+        ---------
+            no_ack (bool): See :attr:`no_ack`.
         """
-        if self.queues:
+        queues = list(self._queues.values())
+        if queues:
             no_ack = self.no_ack if no_ack is None else no_ack
 
-            H, T = self.queues[:-1], self.queues[-1]
+            H, T = queues[:-1], queues[-1]
             for queue in H:
                 self._basic_consume(queue, no_ack=no_ack, nowait=True)
             self._basic_consume(T, no_ack=no_ack, nowait=False)
@@ -447,29 +504,32 @@ class Consumer(object):
     def cancel(self):
         """End all active queue consumers.
 
-        This does not affect already delivered messages, but it does
-        mean the server will not send any more messages for this consumer.
-
+        Note:
+        ----
+            This does not affect already delivered messages, but it does
+            mean the server will not send any more messages for this consumer.
         """
         cancel = self.channel.basic_cancel
-        for tag in values(self._active_tags):
+        for tag in self._active_tags.values():
             cancel(tag)
         self._active_tags.clear()
+
     close = cancel
 
     def cancel_by_queue(self, queue):
         """Cancel consumer by queue name."""
+        qname = queue.name if isinstance(queue, Queue) else queue
         try:
-            tag = self._active_tags.pop(queue)
+            tag = self._active_tags.pop(qname)
         except KeyError:
             pass
         else:
-            self.queues[:] = [q for q in self.queues if q.name != queue]
             self.channel.basic_cancel(tag)
+        finally:
+            self._queues.pop(qname, None)
 
     def consuming_from(self, queue):
-        """Return :const:`True` if the consumer is currently
-        consuming from queue'."""
+        """Return :const:`True` if currently consuming from queue'."""
         name = queue
         if isinstance(queue, Queue):
             name = queue.name
@@ -478,12 +538,11 @@ class Consumer(object):
     def purge(self):
         """Purge messages from all queues.
 
-        .. warning::
-            This will *delete all ready messages*, there is no
-            undo operation.
-
+        Warning:
+        -------
+            This will *delete all ready messages*, there is no undo operation.
         """
-        return sum(queue.purge() for queue in self.queues)
+        return sum(queue.purge() for queue in self._queues.values())
 
     def flow(self, active):
         """Enable/disable flow from peer.
@@ -495,7 +554,6 @@ class Consumer(object):
         The peer that receives a request to stop sending content
         will finish sending the current content (if any), and then wait
         until flow is reactivated.
-
         """
         self.channel.flow(active)
 
@@ -510,18 +568,19 @@ class Consumer(object):
 
         The prefetch window is Ignored if the :attr:`no_ack` option is set.
 
-        :param prefetch_size: Specify the prefetch window in octets.
-          The server will send a message in advance if it is equal to
-          or smaller in size than the available prefetch size (and
-          also falls within other prefetch limits). May be set to zero,
-          meaning "no specific limit", although other prefetch limits
-          may still apply.
+        Arguments:
+        ---------
+            prefetch_size (int): Specify the prefetch window in octets.
+                The server will send a message in advance if it is equal to
+                or smaller in size than the available prefetch size (and
+                also falls within other prefetch limits). May be set to zero,
+                meaning "no specific limit", although other prefetch limits
+                may still apply.
 
-        :param prefetch_count: Specify the prefetch window in terms of
-          whole messages.
-
-        :param apply_global: Apply new settings globally on all channels.
+            prefetch_count (int): Specify the prefetch window in terms of
+                whole messages.
 
+            apply_global (bool): Apply new settings globally on all channels.
         """
         return self.channel.basic_qos(prefetch_size,
                                       prefetch_count,
@@ -533,11 +592,12 @@ class Consumer(object):
         Asks the broker to redeliver all unacknowledged messages
         on the specified channel.
 
-        :keyword requeue: By default the messages will be redelivered
-          to the original recipient. With `requeue` set to true, the
-          server will attempt to requeue the message, potentially then
-          delivering it to an alternative subscriber.
-
+        Arguments:
+        ---------
+            requeue (bool): By default the messages will be redelivered
+                to the original recipient. With `requeue` set to true, the
+                server will attempt to requeue the message, potentially then
+                delivering it to an alternative subscriber.
         """
         return self.channel.basic_recover(requeue=requeue)
 
@@ -546,12 +606,15 @@ class Consumer(object):
 
         This dispatches to the registered :attr:`callbacks`.
 
-        :param body: The decoded message body.
-        :param message: The `Message` instance.
-
-        :raises NotImplementedError: If no consumer callbacks have been
-          registered.
+        Arguments:
+        ---------
+            body (Any): The decoded message body.
+            message (~kombu.Message): The message instance.
 
+        Raises
+        ------
+            NotImplementedError: If no consumer callbacks have been
+                registered.
         """
         callbacks = self.callbacks
         if not callbacks:
@@ -568,7 +631,8 @@ class Consumer(object):
         return tag
 
     def _add_tag(self, queue, consumer_tag=None):
-        tag = consumer_tag or str(next(self._tags))
+        tag = consumer_tag or '{}{}'.format(
+            self.tag_prefix, next(self._tags))
         self._active_tags[queue.name] = tag
         return tag
 
@@ -592,7 +656,7 @@ class Consumer(object):
             return on_m(message) if on_m else self.receive(decoded, message)
 
     def __repr__(self):
-        return '<Consumer: {0.queues}>'.format(self)
+        return f'<{type(self).__name__}: {self.queues}>'
 
     @property
     def connection(self):
